from enum import StrEnum

class ImageResolution(StrEnum):
   LOW = "low"
   HIGH = "high"
   AUTO = "auto"
   @classmethod
   def to_dict(self) -> str:
       return {item.name: item.value for item in ImageResolution}
   @classmethod
   def content_list(self) -> str:
       return [item.value for item in ImageResolution]




from logyca_ai.utils.constants.content import ContentType, ContentRole
from logyca_ai.utils.constants.image import ImageResolution
from logyca_ai.utils.constants.ocr import OCREngineSettings
from logyca_ai.utils.helpers.content_loaders import save_base64_to_file, save_file_from_url, load_text_from_url, decode_base64_to_str
from logyca_ai.utils.helpers.general_utils import get_random_name_datetime, delete_files_by_modification_hours, get_file_name_extension_from_url
from logyca_ai.utils.helpers.text_extraction_microsoft import extract_text_from_docx_file, extract_text_from_excel_file
from logyca_ai.utils.helpers.text_extraction_pdf import extract_text_from_pdf_file
from pydantic import BaseModel, AliasChoices, Field, model_validator
from typing import Any
import os

class MessageExceptionErrors:
    UNSUPPORTED_FILE_FORMAT="Unsupported file format: {}"
    UNSUPPORTED_IMAGE_FORMAT="Unsupported image format: {}"
    UNSUPPORTED_MICROSOFT_FORMAT="Unsupported microsoft format: {}"
    UNSUPPORTED_PDF_FORMAT="Unsupported pdf format: {}"

class Content(BaseModel):
    system: str = Field(default="Personality, context, purpose.",validation_alias=AliasChoices(ContentRole.SYSTEM))
    messages: list = Field(default=[],validation_alias=AliasChoices("messages"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        tmp = self.__dict__.copy()
        tmp["messages"] = [message.to_dict() for message in tmp["messages"]]
        return tmp

class UserMessage(BaseModel):
    additional_content: Any = Field(default="",validation_alias=AliasChoices("additional_content"))
    type: str = Field(default=ContentType.TEXT,validation_alias=AliasChoices("type"))
    user: str = Field(default="",validation_alias=AliasChoices("user"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__

    @classmethod
    def get_supported_types(cls)->list:        
        return ContentType.get_type_list()

    @classmethod
    def get_default_types(cls)->list:        
        return [ContentType.TEXT]

class AssistantMessage(BaseModel):
    assistant: str = Field(default="",validation_alias=AliasChoices("assistant"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__
    
class ImageFileMessage(BaseModel):
    base64_content_or_url: str = Field(default="",validation_alias=AliasChoices("base64_content_or_url"))
    image_format: str = Field(default="",validation_alias=AliasChoices("image_format"))
    image_resolution: str = Field(default=ImageResolution.AUTO,validation_alias=AliasChoices("image_resolution"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__
    
    def __get_mime_types(self,extension:str=None)->str|dict|None:
        mime_types={
            "bmp":"bmp",
            "gif":"gif",
            "jpeg":"jpeg",
            "jpg":"jpg",
            "png":"png",
            "svg":"svg+xml",
            "webp":"webp",
        }
        if extension is None:
            return mime_types
        else:
            return mime_types.get(extension,None)

    @classmethod
    def get_supported_formats(cls)->list:        
        return [key for key, value in cls().__get_mime_types().items()]
        
    @classmethod
    def get_default_types(cls)->list:        
        return [ContentType.IMAGE_URL,ContentType.IMAGE_BASE64]

    def build_message_content(self)->dict|None:
        if self.image_format == ContentType.IMAGE_URL:
            image_url=self.base64_content_or_url
        else: 
            mime_type = self.__get_mime_types(self.image_format)
            if(mime_type is None): raise ValueError(MessageExceptionErrors.UNSUPPORTED_IMAGE_FORMAT.format(self.image_format))
            image_url=f"data:image/{mime_type};base64,{self.base64_content_or_url}"
        return {
            "type": "image_url",
            "image_url": {
                "url" : image_url, 
                "detail" : str(self.image_resolution)
            }
        }

class PdfFileMessage(BaseModel):
    base64_content_or_url: str = Field(default="",validation_alias=AliasChoices("base64_content_or_url"))
    pdf_format: str = Field(default="",validation_alias=AliasChoices("pdf_format"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__
    
    def __get_pdf_formats(self,extension:str=None)->str|dict|None:
        pdf_formats={
            "pdf":"pdf",
        }
        if extension is None:
            return pdf_formats
        else:
            return pdf_formats.get(extension,None)

    @classmethod
    def get_supported_formats(cls)->list:        
        return [key for key, value in cls().__get_pdf_formats().items()]
        
    @classmethod
    def get_default_types(cls)->list:        
        return [ContentType.PDF_URL,ContentType.PDF_BASE64]

    def build_message_content(self,advanced_image_recognition:bool=False,ocr_engine_path:str=None,output_temp_dir:str=None,cleanup_output_temp_dir_after_hours: int = 24)->str|None:
        """
        Build the supported message list.

        :param content: Content to send to chatgpt, which consists of system and messages.
        :type content: str
        :param advanced_image_recognition: (pdf for now) Indicates whether to perform text recognition on images within the files or documents.
                                If True, OCR techniques will be used to extract text from images.
        :type advanced_image_recognition: bool
        :param ocr_engine_path: Path to the OCR executable. If provided, this path will be used instead of the default.
        :type ocr_engine_path: str, optional
        :param output_temp_dir: Temporary directory for storing output files.
                                If not provided, a default tmp temporary directory in the application root folder will be used.
        :type output_temp_dir: str, optional
        :param cleanup_output_temp_dir_after_hours: Number of hours after which the files in the temporary directory will be deleted on the next call of the function.
        :type cleanup_output_temp_dir_after_hours: int, optional

        :return: Supported message list.
        :rtype: str
        """
        if output_temp_dir is None: output_temp_dir=os.path.abspath(os.path.join(os.getcwd(),OCREngineSettings.TMP_DIR))
        if not os.path.exists(output_temp_dir):
            os.makedirs(output_temp_dir)
        delete_files_by_modification_hours(output_temp_dir,cleanup_output_temp_dir_after_hours)
        pdf_filename = f"{get_random_name_datetime()}.pdf"
        pdf_tmp_to_work = os.path.abspath(os.path.join(output_temp_dir,pdf_filename))
        if self.pdf_format == ContentType.PDF_URL:
            save_file_from_url(self.base64_content_or_url,output_temp_dir,pdf_filename)
            pdf_text=extract_text_from_pdf_file(pdf_tmp_to_work,advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir)
            os.remove(pdf_tmp_to_work)
            return pdf_text
        else:
            if(self.__get_pdf_formats(self.pdf_format) is None): raise ValueError(MessageExceptionErrors.UNSUPPORTED_PDF_FORMAT.format(self.pdf_format))
            save_base64_to_file(self.base64_content_or_url,output_temp_dir,pdf_filename)
            pdf_text=extract_text_from_pdf_file(pdf_tmp_to_work,advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir)
            os.remove(pdf_tmp_to_work)
            return pdf_text            

class PlainTextFileMessage(BaseModel):
    base64_content_or_url: str = Field(default="",validation_alias=AliasChoices("base64_content_or_url"))
    file_format: str = Field(default="",validation_alias=AliasChoices("file_format"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__
    
    def __get_file_formats(self,extension:str=None)->str|dict|None:
        file_formats={
            "txt":"txt",
            "csv":"csv",
        }
        if extension is None:
            return file_formats
        else:
            return file_formats.get(extension,None)

    @classmethod
    def get_supported_formats(cls)->list:        
        return [key for key, value in cls().__get_file_formats().items()]
        
    @classmethod
    def get_default_types(cls)->list:        
        return [ContentType.PLAIN_TEXT_URL,ContentType.PLAIN_TEXT_BASE64]

    def build_message_content(self)->str|None:
        if self.file_format == ContentType.PLAIN_TEXT_URL:
            plain_text=load_text_from_url(self.base64_content_or_url)
            return plain_text
        else:
            if(self.__get_file_formats(self.file_format) is None): raise ValueError(MessageExceptionErrors.UNSUPPORTED_FILE_FORMAT.format(self.file_format))
            plain_text=decode_base64_to_str(self.base64_content_or_url)
            return plain_text

class MicrosoftFileMessage(BaseModel):
    base64_content_or_url: str = Field(default="",validation_alias=AliasChoices("base64_content_or_url"))
    file_format: str = Field(default="",validation_alias=AliasChoices("file_format"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__

    def __get_word_extensions(self,extension:str=None)->str|dict|None:
        file_formats = {
            "docx":"docx"
        }
        if extension is None:
            return file_formats
        else:
            return file_formats.get(extension,None)
    
    def __get_excel_extensions(self,extension:str=None)->str|dict|None:
        file_formats = {
            "xls":"xls",
            "xlsx":"xlsx",
        }
        if extension is None:
            return file_formats
        else:
            return file_formats.get(extension,None)
    
    def __get_file_formats(self,extension:str=None)->str|dict|None:
        file_formats = self.__get_word_extensions() | self.__get_excel_extensions()
        if extension is None:
            return file_formats
        else:
            return file_formats.get(extension,None)

    @classmethod
    def get_supported_formats(cls)->list:        
        return [key for key, value in cls().__get_file_formats().items()]
        
    @classmethod
    def get_default_types(cls)->list:        
        return [ContentType.MS_URL,ContentType.MS_BASE64]

    def build_message_content(self,advanced_image_recognition:bool=False,ocr_engine_path:str=None,output_temp_dir:str=None,cleanup_output_temp_dir_after_hours: int = 24)->str|None:
        """
        Build the supported message list.

        :param content: Content to send to chatgpt, which consists of system and messages.
        :type content: str
        :param advanced_image_recognition: (pdf for now) Indicates whether to perform text recognition on images within the files or documents.
                                If True, OCR techniques will be used to extract text from images.
        :type advanced_image_recognition: bool
        :param ocr_engine_path: Path to the OCR executable. If provided, this path will be used instead of the default.
        :type ocr_engine_path: str, optional
        :param output_temp_dir: Temporary directory for storing output files.
                                If not provided, a default tmp temporary directory in the application root folder will be used.
        :type output_temp_dir: str, optional
        :param cleanup_output_temp_dir_after_hours: Number of hours after which the files in the temporary directory will be deleted on the next call of the function.
        :type cleanup_output_temp_dir_after_hours: int, optional

        :return: Supported message list.
        :rtype: str
        """
        if output_temp_dir is None: output_temp_dir=os.path.abspath(os.path.join(os.getcwd(),OCREngineSettings.TMP_DIR))
        if not os.path.exists(output_temp_dir):
            os.makedirs(output_temp_dir)
        delete_files_by_modification_hours(output_temp_dir,cleanup_output_temp_dir_after_hours)
        if self.file_format == ContentType.MS_URL:
            file_name, file_extension=get_file_name_extension_from_url(self.base64_content_or_url)
            if(self.__get_file_formats(file_extension) is None): raise ValueError(MessageExceptionErrors.UNSUPPORTED_MICROSOFT_FORMAT.format(self.file_format))
            ms_filename = f"{get_random_name_datetime()}.{file_extension}"
            ms_tmp_to_work = os.path.abspath(os.path.join(output_temp_dir,ms_filename))
            save_file_from_url(self.base64_content_or_url,output_temp_dir,ms_filename)
        else:
            ms_filename = f"{get_random_name_datetime()}.{self.file_format}"
            ms_tmp_to_work = os.path.abspath(os.path.join(output_temp_dir,ms_filename))
            if(self.__get_file_formats(self.file_format) is None): raise ValueError(MessageExceptionErrors.UNSUPPORTED_MICROSOFT_FORMAT.format(self.file_format))
            save_base64_to_file(self.base64_content_or_url,output_temp_dir,ms_filename)
            file_extension = self.file_format
        if self.__get_word_extensions(file_extension) is not None:
            file_text=extract_text_from_docx_file(ms_tmp_to_work,advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir)
        elif self.__get_excel_extensions(file_extension) is not None:
            file_text=extract_text_from_excel_file(ms_tmp_to_work,advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir)
        else:
            file_text=""
        os.remove(ms_tmp_to_work)
        return file_text



from logyca_ai.utils.constants.content import ContentRole
from logyca_ai.utils.schemes.input.conversations import Content, ImageFileMessage, PdfFileMessage, PlainTextFileMessage, MicrosoftFileMessage
from logyca_ai.utils.schemes.output.conversations import ConversationAnswer, ConversationUsage
from openai import AsyncAzureOpenAI, AzureOpenAI
from openai.types.completion_usage import CompletionUsage
from starlette import status as http_status

class AzureOpenAIChatGPT():

    def __init__(self,azure_endpoint:str,api_key:str,api_version:str) -> None:
        self.async_client = AsyncAzureOpenAI(
            azure_endpoint=f"https://{azure_endpoint}.openai.azure.com",
            api_key=api_key,
            api_version=api_version
        )
        self.sync_client = AzureOpenAI(
            azure_endpoint=f"https://{azure_endpoint}.openai.azure.com",
            api_key=api_key,
            api_version=api_version
        )
    
    def build_conversation_message_list(self,content:Content,advanced_image_recognition:bool=False,ocr_engine_path:str=None,output_temp_dir:str=None,cleanup_output_temp_dir_after_hours: int = 24)->list:
        """
        Build the supported message list.

        :param content: Content to send to chatgpt, which consists of system and messages.
        :type content: str
        :param advanced_image_recognition: Indicates whether to perform text recognition on images within the files or documents.
                                If True, OCR techniques will be used to extract text from images.
        :type advanced_image_recognition: bool
        :param ocr_engine_path: Path to the OCR executable. If provided, this path will be used instead of the default.
        :type ocr_engine_path: str, optional
        :param output_temp_dir: Temporary directory for storing output files.
                                If not provided, a default tmp temporary directory in the application root folder will be used.
        :type output_temp_dir: str, optional
        :param cleanup_output_temp_dir_after_hours: Number of hours after which the files in the temporary directory will be deleted on the next call of the function.
        :type cleanup_output_temp_dir_after_hours: int, optional

        :return: Supported message list.
        :rtype: str

        :example:

        ## Example usage
        ```json
        {
            "system": "",
            messages=[
                    {"role":"system","content":""},
                    {"role":"user","content":""},
                    {"role":"assistant","content":""},
                    {"role":"user","content":""},
                    {"role":"assistant","content":""},
            ]
        }
        ```
        """


        messages= []

        if content.system and content.system!="":
            messages.append({"role":str(ContentRole.SYSTEM),"content":content.system})

        for message in content.messages:
            if isinstance(message,dict) is False:
                message=message.to_dict()
            assistant_message = message.get(ContentRole.ASSISTANT,None)
            if assistant_message is not None:
                messages.append({"role":str(ContentRole.ASSISTANT),"content":assistant_message})
            else:
                user_message = message.get(ContentRole.USER,None)
                if user_message is None:
                    raise ValueError(f"Unsupported role {user_message}")
                else:
                    type_message = message.get("type",None)                

                    if type_message in ImageFileMessage.get_default_types():
                        additional_content = message.get("additional_content",None)
                        messages.append({"role":str(ContentRole.USER),"content":[
                            ImageFileMessage(**additional_content).build_message_content()
                        ]})

                    if type_message in PdfFileMessage.get_default_types():
                        additional_content = message.get("additional_content",None)
                        messages.append({"role":str(ContentRole.USER),"content":
                            PdfFileMessage(**additional_content).build_message_content(advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir,cleanup_output_temp_dir_after_hours=cleanup_output_temp_dir_after_hours)
                        })

                    if type_message in PlainTextFileMessage.get_default_types():
                        additional_content = message.get("additional_content",None)
                        messages.append({"role":str(ContentRole.USER),"content":
                            PlainTextFileMessage(**additional_content).build_message_content()
                        })                        

                    if type_message in MicrosoftFileMessage.get_default_types():
                        additional_content = message.get("additional_content",None)
                        messages.append({"role":str(ContentRole.USER),"content":
                            MicrosoftFileMessage(**additional_content).build_message_content(advanced_image_recognition=advanced_image_recognition,ocr_engine_path=ocr_engine_path,output_temp_dir=output_temp_dir,cleanup_output_temp_dir_after_hours=cleanup_output_temp_dir_after_hours)
                        })                        

                    if user_message is not None and user_message!="":
                        messages.append({"role":str(ContentRole.USER),"content":user_message})


        return messages

    async def conversation_async(self,
        model:str,
        messages:list,
        limit_tokens_answer:int=4000,
        temperature:float=0.7,
        top_p:float=0.95)->tuple[int,ConversationAnswer]:
        """Description
        :return int,str: Http Status Code Response, Message
        
        References:

        - https://learn.microsoft.com/en-us/azure/ai-services/openai/how-to/migration
        
        """
        try:
            completion = await self.async_client.chat.completions.create(
                model=model,
                messages=messages,
                temperature=temperature, # The closer you get to 1 the answer changes, if you want the same answer you should get closer to zero
                max_tokens=limit_tokens_answer,
                top_p=top_p, # 0.1 means only the tokens comprising the top 10% probability mass are considered. Default 1 => 100% probability mass are considered
                frequency_penalty=0,
                presence_penalty=0,
                stop=None
            )
            response=completion.model_dump()
            usage:CompletionUsage = completion.usage
            conversation_response = ConversationAnswer()
            conversation_response.assistant = str(response['choices'][0]['message']['content']).strip()
            conversation_response.usage_data = ConversationUsage(**usage.__dict__)
            return http_status.HTTP_200_OK,conversation_response
        except Exception as e:
            return http_status.HTTP_429_TOO_MANY_REQUESTS,str(e)

    def conversation_sync(self,
        model:str,
        messages:list,
        limit_tokens_answer:int=4000,
        temperature:float=0.7,
        top_p:float=0.95)->tuple[int,ConversationAnswer]:
        """Description
        :return int,str: Http Status Code Response, Message
        
        References:

        - https://learn.microsoft.com/en-us/azure/ai-services/openai/how-to/migration
        
        """
        try:
            completion = self.sync_client.chat.completions.create(
                model=model,
                messages=messages,
                temperature=temperature, # The closer you get to 1 the answer changes, if you want the same answer you should get closer to zero
                max_tokens=limit_tokens_answer,
                top_p=top_p, # 0.1 means only the tokens comprising the top 10% probability mass are considered. Default 1 => 100% probability mass are considered
                frequency_penalty=0,
                presence_penalty=0,
                stop=None
            )
            response=completion.model_dump()
            usage:CompletionUsage = completion.usage
            conversation_response = ConversationAnswer()
            conversation_response.assistant = str(response['choices'][0]['message']['content']).strip()
            conversation_response.usage_data = ConversationUsage(**usage.__dict__)
            return http_status.HTTP_200_OK,conversation_response
        except Exception as e:
            return http_status.HTTP_429_TOO_MANY_REQUESTS,str(e)




from enum import StrEnum
import base64
import os
import requests

class ContentLoadersExceptionErrors(StrEnum):
    DECODING_ERROR = "Could not decode base64 string {}"
    DECODING_ERROR_COMMON_ENCODINGS = "Could not decode base64 string with common encodings {}"
    ERROR_DOWNLOADING_FILE = "Error downloading file {}"
    ERROR_LOADING_FILE_CONTENT = "Error loading file content {}"
    ERROR_SAVING_FILE = "Error saving file {}"

def get_base64_from_file(image_full_path):
    with open(image_full_path, "rb") as image_file:
        return base64.b64encode(image_file.read()).decode('utf-8')

def save_base64_to_file(base64_string: str, output_folder: str, filename: str):
    binary_data = base64.b64decode(base64_string)
    output_path = os.path.join(output_folder, filename)
    with open(output_path, 'wb') as file:
        file.write(binary_data)

def save_file_from_url(url: str, output_folder: str, filename: str):
    try:
        response = requests.get(url)
        response.raise_for_status()
        output_path = os.path.join(output_folder, filename)
        with open(output_path, 'wb') as file:
            file.write(response.content)
    except requests.exceptions.RequestException as e:
        raise RuntimeError(ContentLoadersExceptionErrors.ERROR_DOWNLOADING_FILE.format(e))
    except Exception as e:
        print(f"Error al guardar el archivo: {e}")

def load_text_from_url(url: str) -> str:
    try:
        response = requests.get(url)
        response.raise_for_status()
        data = response.text
        return data
    except requests.exceptions.RequestException as e:
        raise RuntimeError(ContentLoadersExceptionErrors.ERROR_SAVING_FILE.format(e))
    except Exception as e:
        raise RuntimeError(ContentLoadersExceptionErrors.ERROR_LOADING_FILE_CONTENT.format(e))

def decode_base64_to_str(encoded_str: str) -> str:
    common_encodings = ['utf-8', 'iso-8859-1', 'latin-1', 'ascii', 'utf-16', 'utf-32']
    try:
        decoded_bytes = base64.b64decode(encoded_str)
        for encoding in common_encodings:
            try:
                return decoded_bytes.decode(encoding)
            except UnicodeDecodeError:
                continue
        raise UnicodeDecodeError(ContentLoadersExceptionErrors.DECODING_ERROR_COMMON_ENCODINGS.format(common_encodings))
    except Exception as e:
        raise ValueError(ContentLoadersExceptionErrors.DECODING_ERROR.format(e))


from enum import StrEnum
import os

class OCREngine(StrEnum):
    WINDOWS_PATH_01     = "C:\\Program Files\\Tesseract-OCR\\tesseract.exe"
    WINDOWS_PATH_02     = "C:\\Program Files (x86)\\Tesseract-OCR\\tesseract.exe"
    LINUX_PATH_01       = "/usr/bin/tesseract"
    LINUX_PATH_02       = "/usr/local/bin/tesseract"
    
    @classmethod
    def get_binary_path(cls):
        for location in cls:
            if os.path.exists(location):
                return str(location)
        return None

class OCREngineSettings(StrEnum):
    TMP_DIR="tmp"



from docx import Document # python-docx
from io import BytesIO
from logyca_ai.utils.constants.ocr import OCREngine, OCREngineSettings
from openpyxl import load_workbook
from PIL import Image  # Pillow
import os
import pytesseract

def extract_text_from_docx_file(filename_full_path:str,advanced_image_recognition:bool=False,ocr_engine_path:str=None,output_temp_dir:str=None):
    """
    Extracts text from a DOCX file.

    :param filename_full_path: Full path to the DOCX file from which to extract text.
    :type filename_full_path: str
    :param advanced_image_recognition: Indicates whether to perform text recognition on images within the DOCX.
                               If True, OCR techniques will be used to extract text from images.
    :type advanced_image_recognition: bool
    :param ocr_engine_path: Path to the OCR executable. If provided, this path will be used instead of the default.
    :type ocr_engine_path: str, optional
    :param output_temp_dir: Temporary directory for storing output files.
                            If not provided, a default tmp temporary directory in the application root folder will be used.
    :type output_temp_dir: str, optional

    :return: Extracted text from the DOCX file.
    :rtype: str

    :raises FileNotFoundError: If the specified DOCX file is not found.
    :raises ValueError: If the OCR path is invalid.
    """

    if advanced_image_recognition:
        if ocr_engine_path is None:
            pytesseract.pytesseract.tesseract_cmd=OCREngine.get_binary_path()
        else:
            pytesseract.pytesseract.tesseract_cmd=ocr_engine_path

    doc = Document(filename_full_path)

    text = ""
    if output_temp_dir is None:
        output_temp_dir=os.path.abspath(os.path.join(os.path.dirname(__file__),OCREngineSettings.TMP_DIR))
    if not os.path.exists(output_temp_dir):
        os.makedirs(output_temp_dir)

    for paragraph in doc.paragraphs:
        text += paragraph.text + "\n"

    if advanced_image_recognition:
        for rel in doc.inline_shapes:
            if rel.type.value == 3:  # Type 3 corresponds to images
                rId = rel._inline.graphic.graphicData.pic.blipFill.blip.embed
                image_part = doc.part.related_parts[rId]
                image_bytes = image_part.image.blob

                image = Image.open(BytesIO(image_bytes))
                ocr_text = pytesseract.image_to_string(image)
                text += ocr_text

    return text

def extract_text_from_excel_file(filename_full_path: str, advanced_image_recognition: bool = False, ocr_engine_path: str = None, output_temp_dir: str = None):
    """
    Extracts text from an Excel file including all sheets and any embedded images.

    :param filename_full_path: Full path to the Excel file from which to extract text.
    :param advanced_image_recognition: Indicates whether to perform OCR on images within the Excel file.
    :param ocr_engine_path: Path to the OCR executable.
    :param output_temp_dir: Temporary directory for storing output files.
    :return: Extracted text from the Excel file.
    :rtype: str
    """
    # Set OCR engine path
    if advanced_image_recognition:
        if ocr_engine_path is None:
            pytesseract.pytesseract.tesseract_cmd = OCREngine.get_binary_path()
        else:
            pytesseract.pytesseract.tesseract_cmd = ocr_engine_path

    if output_temp_dir is None:
        output_temp_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), OCREngineSettings.TMP_DIR))
    if not os.path.exists(output_temp_dir):
        os.makedirs(output_temp_dir)

    text = ""

    workbook = load_workbook(filename_full_path, data_only=True)
    for sheet_name in workbook.sheetnames:
        sheet = workbook[sheet_name]
        text += f"Sheet: {sheet_name}\n"
        
        for row in sheet.iter_rows(values_only=True):
            for cell in row:
                if cell:
                    text += str(cell) + " "

        if advanced_image_recognition:
            for image in sheet._images:
                img = Image.open(BytesIO(image._data()))
                ocr_text = pytesseract.image_to_string(img)
                text += ocr_text

        text += "\n"

    return text


from pydantic import BaseModel, AliasChoices, Field, model_validator

class ConversationUsage(BaseModel):
    completion_tokens: int = Field(default=0,validation_alias=AliasChoices("completion_tokens"))
    prompt_tokens: int = Field(default=0,validation_alias=AliasChoices("prompt_tokens"))
    total_tokens: int = Field(default=0,validation_alias=AliasChoices("total_tokens"))
    
    @model_validator(mode="before")
    def check_keys(cls, values):
        return values

    def to_dict(self)->dict:
        return self.__dict__

class ConversationAnswer(BaseModel):
    assistant:str = Field(default="")
    usage_data:ConversationUsage = Field(default=ConversationUsage())

    def to_dict(self)->dict:
        tmp = self.__dict__.copy()
        tmp["usage_data"] = tmp["usage_data"].to_dict()
        return tmp
